import { MaybeFactory, MaybePromise } from '../utils/types';
import { ServiceEventEmitter, ServiceEventSubscriber } from './event';
import { ServiceInstance } from './service-instance';
import { InferServiceID, Service, ServiceInterface } from './service-interface';
import { InferModuleID, ServiceModule, ServiceModuleRegistry } from './service-module-registry';
import { ServiceRegistry } from './service-registry';
export interface ServiceInstanceBuilder_ServiceFactory<T extends ServiceInterface = any> {
    id: InferServiceID<T>;
    factory: MaybeFactory<Service<T>>;
}
export declare function serviceFactory<T extends ServiceInterface = ServiceInterface<never, never, never, never>>(id: InferServiceID<T>, factory: MaybeFactory<Service<T>>): ServiceInstanceBuilder_ServiceFactory;
export interface ServiceInstanceBuilder_ModuleFactory<T extends ServiceModule = any> {
    id: InferModuleID<T>;
    factory: MaybeFactory<T>;
}
export declare function moduleFactory<T extends ServiceModule>(id: InferModuleID<T>, factory: MaybeFactory<T>): ServiceInstanceBuilder_ModuleFactory;
export interface ServiceInstanceBuilder_ServiceGroup {
    services: ServiceInstanceBuilder_ServiceFactory[];
    modules?: ServiceInstanceBuilder_ModuleFactory[];
    eventSubscribers?: (serviceId: string) => MaybePromise<ServiceEventSubscriber>;
    eventEmitters?: (serviceId: string) => MaybePromise<ServiceEventEmitter>;
}
export interface ServiceInstanceBuilder_Default {
    modules?: ServiceInstanceBuilder_ModuleFactory[];
    eventSubscribers?: (serviceId: string) => MaybePromise<ServiceEventSubscriber>;
    eventEmitters?: (serviceId: string) => MaybePromise<ServiceEventEmitter>;
}
export declare class ServiceInstanceBuilder {
    _serviceRegistry: ServiceRegistry;
    _serviceModuleRegistries: {
        [serviceId: string]: ServiceModuleRegistry;
    };
    _defaultModuleRegistry: ServiceModuleRegistry;
    _defaultEventSubscribers?: (serviceId: string) => MaybePromise<ServiceEventSubscriber>;
    _defaultEventEmitters?: (serviceId: string) => MaybePromise<ServiceEventEmitter>;
    constructor();
    default(defaultFactories: ServiceInstanceBuilder_Default): this;
    appendServices(group: ServiceInstanceBuilder_ServiceGroup): this;
    private _resolveServices;
    createInstance(): Promise<ServiceInstance>;
}
//# sourceMappingURL=service-instance-builder.d.ts.map