import { Collection, ObjectId, WithId } from 'mongodb';
import { BaseService, ServiceInterface } from '../../../core';
declare type MongoDoc<T> = {
    _id: ObjectId;
    value: T;
};
export interface EntityComponent<T> {
    id: string;
    value: T;
}
export interface EntityComponentServiceOptions {
    index?: 'non-unique' | 'unique';
}
interface EntityComponentServiceInterface_Functions<T> {
    getByIds(ids: string[]): Promise<(T | undefined)[]>;
    get(id: string): Promise<T | undefined>;
}
export declare type EntityComponentServiceInterface<T> = {
    functions: EntityComponentServiceInterface_Functions<T>;
    errors: {
        VALUE_NOT_UNIQUE: {
            value: T;
        };
    };
    events: {
        SETTED: {
            id: string;
            value: T;
        };
    };
};
export declare class EntityComponentService<TServiceInf extends ServiceInterface & EntityComponentServiceInterface<T> = any, T = any> extends BaseService<TServiceInf> {
    private _module;
    private _index?;
    private _collection;
    get collection(): Collection<MongoDoc<T>>;
    constructor(options?: EntityComponentServiceOptions);
    _createIndexes(): Promise<void>;
    protected _parseEntityComponent(doc: WithId<MongoDoc<T>>): EntityComponent<T>;
    protected _parseDocId(id: string): string | ObjectId;
    protected _toEntityId(docId: ObjectId | string): string;
    protected _get(id: string): Promise<T | undefined>;
    protected _set(id: string, value: T | undefined): Promise<void>;
    protected _getByIds(ids: string[]): Promise<(T | undefined)[]>;
    protected _getIdByValue(value: T): Promise<string | undefined>;
    protected _getIdsByValue(value: T): Promise<string[]>;
    protected _getAll(): Promise<EntityComponent<T>[]>;
}
export {};
//# sourceMappingURL=entity-component.d.ts.map